# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2021 CERN.
# Copyright (C) 2020-2021 Northwestern University.
#
# Marshmallow-Utils is free software; you can redistribute it and/or modify
# it under the terms of the MIT License; see LICENSE file for more details.

"""Version information for Marshmallow-Utils.

This file is imported by ``marshmallow_utils.__init__``,
and parsed by ``setup.py``.
"""

__version__ = '0.5.5'
